
LIBRARY ieee;
USE ieee.math_real.ALL;
USE ieee.electrical_systems.ALL;

ENTITY mydiode IS
  GENERIC (
           AREA : REAL := 1.0;
           BV   : REAL := 100.0;
           CJO  : REAL := 0.4E-11;
           GMIN : REAL := 1.0E-12;
           IBV  : REAL := 0.1E-3;
           ISS  : REAL := 0.268E-8;
           RS   : REAL := 0.6E0;
           K    : REAL := 0.138062E-22;
           Q    : REAL := 0.160219E-18
          );
  PORT (
        TERMINAL PIN, POUT : ELECTRICAL
       );
END ENTITY mydiode;


ARCHITECTURE dae OF mydiode IS

  QUANTITY VIN  ACROSS IIN  THROUGH PIN;
  QUANTITY VOUT ACROSS IOUT THROUGH POUT;

  -- Sequential Vars
  QUANTITY VD : REAL;
  QUANTITY ID : REAL;
  QUANTITY CJ : REAL;

  -- Simultaneous Vars
  QUANTITY dIDdt : REAL;
  QUANTITY dVDdt : REAL;

  -- If-functions
  FUNCTION iffunc1(VDi : REAL)
    RETURN REAL IS
      VARIABLE res : REAL;
    BEGIN
      IF VDi < 0.25 THEN
        res := 1.0/((1.0 -2.0*VDi)**(0.333));
      ELSE
        res := 2.51926*(0.3335+0.666*VDi);
      END IF;
    RETURN res;
  END;

  -- Sequential equations
  TYPE QR IS RECORD
    VDv : REAL;
    IDv : REAL;
    CJv : REAL;
  END RECORD;

  FUNCTION proceqs(
    VINi   : REAL;
    IINi   : REAL;
    VOUTi  : REAL;
    IOUTi  : REAL;
    VDi    : REAL;
    IDi    : REAL;
    CJi    : REAL;
    dIDdti : REAL;
    dVDdti : REAL
  ) RETURN QR IS
     VARIABLE VINv   : REAL := VINi;
     VARIABLE IINv   : REAL := IINi;
     VARIABLE VOUTv  : REAL := VOUTi;
     VARIABLE IOUTv  : REAL := IOUTi;
     VARIABLE VDv    : REAL := VDi;
     VARIABLE IDv    : REAL := IDi;
     VARIABLE CJv    : REAL := CJi;
     VARIABLE dIDdtv : REAL := dIDdti;
     VARIABLE dVDdtv : REAL := dVDdti;
  BEGIN
    VDv := VINv-VOUTv-IINv*RS/AREA;
    IDv := GMIN*VDv+AREA*(ISS*(-1.0+exp(0.181069E-2*VDv*Q/K))-IBV*exp((-0.333167E-2)*(BV+VDv)*Q/K));
    CJv := AREA*CJO*iffunc1(VDv);
    RETURN (vdv, idv, cjv);
  END;


BEGIN
  
  -- Assignment for sequential equations
  (VD, ID, CJ) == proceqs(VIN, IIN, VOUT, IOUT, VD, ID, CJ, dIDdt, dVDdt) TOLERANCE "Current";

  -- Simultaneous Equations
  IIN == -IOUT TOLERANCE "Current";
  -ID-0.115E-7*dIDdt+IIN-CJ*dVDdt == 0.0 TOLERANCE "Current";
  dIDdt == ID'dot TOLERANCE "Current";
  dVDdt == VD'dot TOLERANCE "Current";

END ARCHITECTURE dae;
